#include "project-defs.h"
#include "stepper.h"

static volatile bool doStep = false;

__attribute__((interrupt("WCH-Interrupt-fast")))
void TIM2_IRQHandler() {
	if (TIM_GetITStatus(TIM2, TIM_IT_Update) != RESET) {
		doStep = true;
		TIM_ClearITPendingBit(TIM2, TIM_IT_Update);
	}
}

int main() {
	GPIO_InitTypeDef gpioInit = { 0 };

	// On configure les entrées des boutons poussoirs.
	// Rappel : ne rien connecter à PD1, elle est utilisée par SWIO.
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD, ENABLE);
	gpioInit.GPIO_Pin = GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_4;
	gpioInit.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_Init(GPIOD, &gpioInit);

	// On configure les sorties connectées au module L9110S.
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);
	gpioInit.GPIO_Pin = GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6;
	gpioInit.GPIO_Mode = GPIO_Mode_Out_PP;
	gpioInit.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOC, &gpioInit);

	// On configure le timer.
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);

	TIM_TimeBaseInitTypeDef timeBaseInit = { 0 };
	timeBaseInit.TIM_Period = ((uint16_t) (MCU_FREQ / PRESCALER / STEP_FREQ - 1));
	timeBaseInit.TIM_Prescaler = PRESCALER - 1;
	timeBaseInit.TIM_ClockDivision = TIM_CKD_DIV1;
	timeBaseInit.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseInit(TIM2, &timeBaseInit);

	NVIC_InitTypeDef nvicInit = { 0 };
	nvicInit.NVIC_IRQChannel = TIM2_IRQn;
	nvicInit.NVIC_IRQChannelPreemptionPriority = 1;
	nvicInit.NVIC_IRQChannelSubPriority = 0;
	nvicInit.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&nvicInit);

	TIM_ITConfig(TIM2, TIM_IT_Update , ENABLE);
	TIM_Cmd(TIM2, ENABLE);

	bool greenButton = false;
	stepperInit(greenButton);

	while (1) {
		uint16_t buttons = GPIO_ReadInputData(GPIOD);
		// Le bouton vert est relié à PD4, qui est à l'état bas s'il est appuyé.
		bool newGreenButton = !(buttons & GPIO_Pin_4);
		// Le bouton bleu est relié à PD3, qui est à l'état bas s'il est appuyé.
		bool blueButton = !(buttons & GPIO_Pin_3);
		// Le bouton rouge est relié à PD2, qui est à l'état bas s'il est appuyé.
		bool redButton = !(buttons & GPIO_Pin_2);

		if (greenButton != newGreenButton) {
			// Le mode de pilotage du moteur a changé.
			greenButton = newGreenButton;
			// On configure le nouveau mode.
			stepperInit(greenButton);
		}

		if (doStep) {
			doStep = false;

			if (blueButton || redButton) {
				// Un des 2 boutons de direction est appuyé, passons au pas suivant.
				// Si le bouton rouge est appuyé, on avance, sinon on recule.
				stepperNext(redButton);
			}
		}
	}
}
