#include "project-defs.h"
#include "stepper.h"

// Les valeurs ci-dessous tiennent dj compte du fait que l'tat commence  PC3.
const uint8_t WAVE_DRIVE[] = { 0x08, 0x40, 0x10, 0x20, };
const uint8_t FULL_STEP_DRIVE[] = { 0x28, 0x48, 0x50, 0x30, };
const uint8_t HALF_STEP_DRIVE[] = { 0x08, 0x48, 0x40, 0x50, 0x10, 0x30, 0x20, 0x28, };

static struct {
	const uint8_t *drive;
	int steps;
	int currentStep;
} stepperState;

void stepperInit(bool greenButton) {
	// On met le moteur hors tension.
	GPIO_Write(GPIOC, 0);

	// Si le bouton vert est appuy, on est en mode half-step, sinon full-step.
	stepperState.drive = greenButton ? HALF_STEP_DRIVE : FULL_STEP_DRIVE;
	stepperState.steps = greenButton ? sizeof(HALF_STEP_DRIVE) : sizeof(FULL_STEP_DRIVE);

	// On veut dmarrer au dbut du tableau au prochain pas (ou  la fin si on recule).
	stepperState.currentStep = stepperState.steps;
}

void stepperNext(bool moveForward) {
	// On calcule l'indice du prochain pas dans le tableau en cours.
	if (moveForward) {
		stepperState.currentStep++;

		if (stepperState.currentStep >= stepperState.steps) {
			stepperState.currentStep = 0;
		}
	} else {
		stepperState.currentStep--;

		if (stepperState.currentStep < 0) {
			stepperState.currentStep = stepperState.steps - 1;
		}
	}

	// On pilote le L9110S.
	GPIO_Write(GPIOC, stepperState.drive[stepperState.currentStep]);
}
